addpath 'UGM/'
addpath 'UGM/compiled/'
addpath 'UGM/decode/'
addpath 'UGM/examples/'
addpath 'UGM/infer/'
addpath 'UGM/KPM/'
addpath 'UGM/mex/'
addpath 'UGM/minConf_2009/'
addpath 'UGM/minFunc_2009/'
addpath 'UGM/misc/'
addpath 'UGM/sample/'
addpath 'UGM/sub/'
addpath 'UGM/train/'

% Experiment Settings
sample_num = 10000; %sample number for step 1 (estimating absolute LS)
sample_num2 = 1; %sample number for step 2 (estimating negative ratio)
burnin1 = 10; % burnin for step 1
burnin2 = 10; % burnin for step 2
cooling_length = 4; % cooling length for step 1

% GM is defined on K4 graph 
nStates = 2;
nNodes = 4;
nEdges = 6;
adj = [0, 1, 1, 1;
    1, 0, 1, 1;
    1, 1, 0, 1;
    1, 1, 1, 0];
edgeStruct = UGM_makeEdgeStruct(adj,nStates);
edgeStruct.adj = adj;
edgeStruct.nComponents=1;

% GM have no external field, interaction strength ~ N(0.5, 0.01)
nodeMagneticField = zeros(nNodes,1);
edgeInteraction =  normrnd(0.5,0.01, [1,nEdges]);
[nodePot, edgePot] = IsingtoUGM(nodeMagneticField, edgeInteraction);

% Compute true partition function
[nodeBel,edgeBel,logZ] = UGM_Infer_Exact(nodePot,edgePot,edgeStruct);

% Run Belief Propagation
edgeStruct.maxIter = 1000;
edgeStruct.useMex = 0;
[nodeBelBethe,edgeBelBethe,logZBethe] = UGM_Infer_LBP(nodePot,edgePot,edgeStruct);

% Run MCMC-BP-2reg
fprintf('doing TwoLS\n');
[logTwoLS, ~] = TwoLoopSeriesMCMC(edgeStruct, nodeBelBethe, edgeBelBethe,cooling_length, sample_num,sample_num2, burnin1, burnin2);

% Run MCMC-BP-Whole
fprintf('doing Whole\n');
[logWholeLS, ~] = WholeLoopSeriesMCMC( edgeStruct, nodeBelBethe, edgeBelBethe, cooling_length, sample_num,sample_num2, burnin1,burnin2);

% Results
fprintf('true: %d\n',logZ);
fprintf('2-reg MCMCBP: %d\n',logZBethe+logTwoLS);
fprintf('full MCMCBP: %d\n', logZBethe+logWholeLS);












