function [ WholeLS ,est_ratio] = WholeLoopSeriesMCMC( edgeStruct, nodeBelLBP, edgeBelLBP, cooling_length, sample_num,sample_num2, burnin1, burnin2 )
nEdges = double(edgeStruct.nEdges);
edgeEnds = double(edgeStruct.edgeEnds);
nNodes = double(edgeStruct.nNodes);
nComponents = double(edgeStruct.nComponents);
cooling_length = cooling_length - 1;

% compute edge weights based on BP fixed points
logEdgeWeights = zeros(nEdges,1);
for e = 1:nEdges
    tau_u = nodeBelLBP(edgeEnds(e,1),2);
    tau_v = nodeBelLBP(edgeEnds(e,2),2);
    tau_uv = edgeBelLBP(2,2,e);
    logEdgeWeights(e,1) =  abs(log(abs((tau_uv - tau_u*tau_v) / (tau_u *tau_v *(1-tau_u) * (1-tau_v))^(0.5))));
end

% Find cycle basis and all-pair path set
G = sparse([edgeEnds(:,1); edgeEnds(:,2)], [edgeEnds(:,2); edgeEnds(:,1)], [logEdgeWeights; logEdgeWeights]);
X_BASIS = zeros(nEdges, (nNodes * nNodes - nNodes)/2 + nEdges - nNodes + nComponents);
idx = 1;
for v = 1:nNodes
    [~,PATH,~] = graphshortestpath(G,v, 'METHOD', 'Bellman-Ford'); 
    for u = v+1: nNodes
        X_BASIS(:, idx) =  TrailtoVector(PATH{u},edgeStruct, 'path');
        idx = idx + 1;
    end
end
CB = cyclebasis(G, 'path');
for i = 1:size(CB,2)
    X_BASIS(:, idx) =  TrailtoVector(CB{i},edgeStruct, 'cycle');
    idx = idx + 1;
end

% run step 1 (estimating absolute loop series)
beta= @(k)(1 + (1+ cooling_length - k)/cooling_length);
one_cnt = 1;
X = zeros(nEdges,1);
maxX = X;
flagj = 0;

% At first cooling length, we want to find a large enough element maxX, for
% initial point of MC
for j = 1:sample_num
    X= LoopMC(edgeStruct,nodeBelLBP, edgeBelLBP,beta(1), X_BASIS, burnin1, X);
    if isequal(X, maxX)
        one_cnt = one_cnt+1;
    elseif abs(LoopTerm(edgeStruct,nodeBelLBP, edgeBelLBP, 1, X)) > abs(LoopTerm(edgeStruct,nodeBelLBP, edgeBelLBP, 1, maxX))
        one_cnt = 1;
        flagj = j;
        maxX = X;
    end
end
approxProd = abs(LoopTerm(edgeStruct,nodeBelLBP, edgeBelLBP, beta(1), maxX)) * (sample_num - flagj + 1) / one_cnt;

% run the rest of MCMC
for i = 1:cooling_length
    fprintf('Schedule: %d / %d\n',i, cooling_length);
    approxRatio = 0;
    for j = 1:sample_num
        X= LoopMC(edgeStruct,nodeBelLBP, edgeBelLBP,beta(i), X_BASIS, burnin1, X);
        approxRatio = approxRatio + (abs(LoopTerm(edgeStruct,nodeBelLBP, edgeBelLBP, 1, X))^(beta(i+1) - beta(i)) / sample_num);
    end
    approxProd = approxProd *approxRatio;
end

% run step 2 (estimating negative ratio)
minus_ratio = 0;
X = maxX;
for j = 1:sample_num2
    X = LoopMC(edgeStruct,nodeBelLBP, edgeBelLBP,1, X_BASIS, burnin2, X);
    if  LoopTerm(edgeStruct,nodeBelLBP, edgeBelLBP, 1, X)< 0
        minus_ratio = minus_ratio + 1/sample_num2;
    end
end
est_ratio = (1-2*minus_ratio);

%estimated ratio cannot be negative
if est_ratio < 1/sample_num2
    est_ratio = 1/sample_num2;
end

WholeLS = log(est_ratio *approxProd);
end


function X = TrailtoVector(path, edgeStruct, type)
nEdges = double(edgeStruct.nEdges);
edgeEnds = double(edgeStruct.edgeEnds);
len = size(path,2);
X = zeros(nEdges, 1);
for i = 1:len-1
    e = path(1,i:i+1);
    for j = 1:nEdges
        if( ~isempty(find(edgeEnds(j,:)== e(1,1),1)) &&  ~isempty(find(edgeEnds(j,:)== e(1,2),1)))
            X(j, 1) = 1;
        end
    end
end
if strcmp(type, 'cycle')
    e = path(1,[len, 1]);
    for j = 1:nEdges
        if( ~isempty(find(edgeEnds(j,:)== e(1,1),1)) &&  ~isempty(find(edgeEnds(j,:)== e(1,2),1)))
            X(j, 1) = 1;
        end
    end
end
end
